/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package dao;

import entidad.Producto;
import java.util.List;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 *
 * @author Jose Luis
 */
public class ProductoDAO {

    private Session session;
    private Transaction transaction;

    public ProductoDAO() {
    }

    protected void iniciaOperacion() throws HibernateException {
        session = HibernateUtil.getSessionFactory().openSession();
        transaction = session.beginTransaction();
    }

    protected Session getSession() {
        return session;
    }

    protected Transaction getTransaction() {
        return transaction;
    }

    protected void manejaExcepcion(HibernateException he) throws HibernateException {
        ProductoDAO proDAO = new ProductoDAO() {
        };
        proDAO.getTransaction().rollback();
        throw new HibernateException("Ocurrió un error en la base de datos", he);
    }

    public boolean insertarProducto(Producto pro) {

        ProductoDAO proDAO = new ProductoDAO() {
        };
        try {

            proDAO.iniciaOperacion();
            proDAO.getSession().save(pro);
            proDAO.getTransaction().commit();

            return true;

        } catch (HibernateException he) {
            proDAO.manejaExcepcion(he);
            throw he;
        } finally {
            proDAO.getSession().close();
        }
    }

    public static List<Producto> obtenListaProductos(int idCategoria) throws HibernateException {
        List<Producto> listaProductos = null;
        ProductoDAO proDAO = new ProductoDAO() {
        };
        try {
            proDAO.iniciaOperacion();
            listaProductos = proDAO.getSession().createQuery("from Producto p WHERE p.idCategoria=" + idCategoria + "ORDER BY nombreProducto").list();
        } catch (HibernateException he) {
            proDAO.manejaExcepcion(he);
            throw he;
        } finally {
            proDAO.getSession().close();
        }

        return listaProductos;
    }

    public static List<Producto> obtenProductosAleatorios() throws HibernateException {
        List<Producto> listaProductos = null;
        ProductoDAO proDAO = new ProductoDAO() {
        };
        try {
            proDAO.iniciaOperacion();
            Query q = proDAO.getSession().createQuery("FROM Producto ORDER BY RAND()").setMaxResults(10);
            listaProductos = q.list();
        } catch (HibernateException he) {
            proDAO.manejaExcepcion(he);
            throw he;
        } finally {
            proDAO.getSession().close();
        }

        return listaProductos;
    }

    //Obtener producto cuando no tengo el id
    public int obtenIdProducto(Producto producto) throws HibernateException {
        Producto temp = new Producto();
        ProductoDAO proDAO = new ProductoDAO() {
        };

        try {
            proDAO.iniciaOperacion();
            Query query = proDAO.getSession().createQuery("from Producto where nombreProducto = :nombre AND descripcion = :desc ");
            query.setParameter("nombre", producto.getNombreProducto());
            query.setParameter("desc", producto.getDescripcion());

            temp = (Producto) query.uniqueResult();
        } catch (HibernateException he) {
            proDAO.manejaExcepcion(he);
            throw he;
        } finally {
            proDAO.getSession().close();
        }

        return temp.getIdProducto();
    }

    //Obtener producto por id
    public static Producto obtenProducto(int idProducto) throws HibernateException {
        Producto producto = null;
        ProductoDAO proDAO = new ProductoDAO() {
        };
        try {
            proDAO.iniciaOperacion();
            producto = (Producto) proDAO.getSession().get(Producto.class, idProducto);
        } catch (HibernateException he) {
            proDAO.manejaExcepcion(he);
            throw he;
        } finally {
            proDAO.getSession().close();
        }
        return producto;
    }

    public static void eliminaProducto(Producto producto) throws HibernateException {
        ProductoDAO proDAO = new ProductoDAO() {
        };
        try {
            proDAO.iniciaOperacion();
            proDAO.getSession().delete(producto);
            proDAO.getTransaction().commit();
        } catch (HibernateException he) {
            proDAO.manejaExcepcion(he);
            throw he;
        } finally {
            proDAO.getSession().close();
        }
    }

    public static void actualizaProducto(Producto producto) throws HibernateException {
        ProductoDAO proDAO = new ProductoDAO() {
        };
        try {
            proDAO.iniciaOperacion();
            proDAO.getSession().update(producto);
            proDAO.getTransaction().commit();
        } catch (HibernateException he) {
            proDAO.manejaExcepcion(he);
            throw he;
        } finally {
            proDAO.getSession().close();
        }
    }

    public List<Producto> busqueda(String busqueda) {

        List<Producto> listaProductos = null;
        ProductoDAO proDAO = new ProductoDAO() {
        };
        try {
            proDAO.iniciaOperacion();
            Query query = proDAO.getSession().createQuery("from Producto where nombreProducto like '%" + busqueda + "%'");

            listaProductos = query.list();
        } catch (HibernateException he) {
            proDAO.manejaExcepcion(he);
            throw he;
        } finally {
            proDAO.getSession().close();
        }

        return listaProductos;
    }

    public List<Producto> stock() {

        List<Producto> listaProductos = null;
        ProductoDAO proDAO = new ProductoDAO() {
        };
        try {
            proDAO.iniciaOperacion();
            Query query = proDAO.getSession().createQuery("FROM Producto ORDER BY stock");

            listaProductos = query.list();
        } catch (HibernateException he) {
            proDAO.manejaExcepcion(he);
            throw he;
        } finally {
            proDAO.getSession().close();
        }

        return listaProductos;
    }
}
